
%% Gillespie algorithm
function [A, t]=Gillespie(A0, getAlpha, e, p, tfin, Max_steps);
% Gillespe Stochastic Simulation Algorithm 
% for simulation of  stochastic processes in Ncomp compartments
%  e(1,i) A(1) + e(2,i) A(2) + ........ + e(Ncomp,i) A(comp) ---> 
%  p(1,i) A(1) + p(2,i) A(2) + ........ + p(Ncomp,i) A(comp)
%    A0 ... Ncomp-by-1 column vector;
%           A0(j) = # of elements in jth compartment at time t=0
%    getAlpha handle to a function that returns propensities of 
%           processes (as a row vector)       
%    e ... Nproc-by-Ncomp matrix describing what enters the process
%    p ... Nproc-by-Ncomp matrix describing products the process
%    tfin ... final time; if the simulation reaches tfin it stops
%    Max_steps ... maximal number of updates the simulation will run
%
%OUTPUT:
%    A ... Nchs-by-N; A(j,m) = # in jth compartment in time [t(m), t(m+1)]
%          A(:,1) == A0(:);
%    t ... 1-by-N; times

% INIT variables and allocate memory for Max_steps steps
Ncomp = size(A0,1);     % number of rows of A0 (number of compartments)
A = zeros(Ncomp,Max_steps); % for tracking compartments
t = zeros(1,Max_steps);     % for time
m = 1;                      %start with the first step
A(:,1) = A0; %everything in the first (1) collumn of A will be set to A0
% main loop
while ((t(m)<tfin) && (m<=Max_steps))
    % (a) generate two randon numbers r1 and r2 uniformly  in (0,1)
    r = rand(1,2);
    % (b) Compute alpha0 = \sum_{proc=1}^Nproc alpha_proc(t),
    %     where alpha_proc(t) = propensity function of proc-th process
    alpha = getAlpha(A(:,m)); % get alpha
    a0 = sum(alpha);
    % (c) The next process takes place at time t+tau, where
    %     tau = 1/alpha0 * log(1/r1)   ... natural logrithm
    if a0 > 0 % if any process is going to happen
        t_next = t(m) + 1/a0 * log(1/r(1)); % determine when
    else % no process is going to happen (now and from now on)
        t_next = tfin; % go to the time limit
    end; %if a0>0
    if t_next>=tfin  % if next process is to occur after the time limit
        t(m+1)=tfin; % it will not occur at all
        A(:,m+1)=A(:,m);
    else %next process within the time limit
        t(m+1) = t_next; % increase the time
        % (d) Determine which process happens. Find proc such that
        %     r2 >= 1/a0 \sum_{i=1}^{proc-1} alpha_i(t)
        %     r2 <  1/a0 \sum_{i=1}^proc     alpha_i(t)
        proc =  find((cumsum(alpha) >= r(2)*a0),1,'first'); % which one
        %     Update the numbers in each compartment
        A(:,m+1) = A(:,m) - e(:,proc) + p(:,proc); % let it happen
    end; %if t_next>=tfin
    m = m + 1; % prepare for the next step
end % while
A = A(:,1:m); % cut zeros at the end of arrays
t = t(1:m);   % cut zeros at the end of arrays
end %Gillespie
