function wasps
% This script uses Gillespie stochastic simulation algorithm to
% simulate two superparasitisng wasps on a patch of hosts
% and plots the history of the system

%%  User defined input:
U = 10;         % initial number of unparasitised hosts
time = 20;      % time both wasps spend on the patch
mu = 1;         % rate of parasitism (hosts per time unit)
sigma = 0.5;    % probability that the latest egg will yield offspring

%% prepare input for gillespie algorithm
function a=Propensities(x)
    % x represents the current # of elements in compartments
    % outputs the alpha values (propensities) for gillespie algorithm
    
    % there are 4 processes with 3 compartments U,V,W
    % U --> V with speed mu
    % U --> W with speed mu
    % V --> W with speed sigma*mu
    % W --> V with speed sigma*mu
    a(1) = mu * (x(1)>0); % wasp (#1) at rate mu (if any U is present)
    a(2) = mu * (x(1)>0); % wasp (#2) at rate mu (if any U is present)
    a(3) = sigma* mu * (x(2)>0); % wasp (#2) at rate sigma*mu (if V>0)
    a(4) = sigma* mu * (x(3)>0); % wasp (#1) at rate sigma*mu (if W>0) 
end;

e = [1, 1, 0, 0;    %U entering process 1, 2, 3, 4
     0, 0, 1, 0;    %V entering process 1, 2, 3, 4
     0, 0, 0, 1];   %W entering process 1, 2, 3, 4

p = [0, 0, 0, 0;    %U resulting from process 1, 2, 3, 4 
     1, 0, 0, 1;    %V resulting from process 1, 2, 3, 4
     0, 1, 1, 0];   %W resulting from process 1, 2, 3, 4
     
A_init = [U; 0; 0];  %U; V; W

%% call the Gillespie algorithm
[Wasps, T]= Gillespie(A_init, @Propensities, e, p, time, 10000);

%% Plots
% the following few lines prepare for stair like drawing of the results
Wasps_stairs=zeros(Ncomp, 2*m-1);
t_stairs=zeros(1, 2*m-1);
Wasps_stairs(:,1:2:2*m-1)=Wasps(:, 1:m);
Wasps_stairs(:,2:2:2*(m-1))=Wasps(:, 1:(m-1));
t_stairs(1:2:2*m-1)=T(1:m);
t_stairs(2:2:2*(m-1))=T(2:m);

figure(1) % evolution of U (unparasitised eggs)
plot(t_stairs,Wasps_stairs(1,:)); 
figure(2) % evolution of V (# of offspring to wasp 1)
plot(t_stairs,Wasps_stairs(2,:)); 
figure(3) % evolution of W (# of offspring to wasp 2)
plot(t_stairs,Wasps_stairs(3,:)); 

%% Gillespie algorithm
function [A, t]=Gillespie(A0, getAlpha, e, p, tfin, Max_steps);
% Gillespe Stochastic Simulation Algorithm 
% for simulation of  stochastic processes in Ncomp compartments
%  e(1,i) A(1) + e(2,i) A(2) + ........ + e(Ncomp,i) A(comp) ---> 
%  p(1,i) A(1) + p(2,i) A(2) + ........ + p(Ncomp,i) A(comp)
%    A0 ... Ncomp-by-1 column vector;
%           A0(j) = # of elements in jth compartment at time t=0
%    getAlpha handle to a function that returns propensities of 
%           processes (as a row vector)       
%    e ... Nproc-by-Ncomp matrix describing what enters the process
%    p ... Nproc-by-Ncomp matrix describing products the process
%    tfin ... final time; if the simulation reaches tfin it stops
%    Max_steps ... maximal number of updates the simulation will run
%
%OUTPUT:
%    A ... Nchs-by-N; A(j,m) = # in jth compartment in time [t(m), t(m+1)]
%          A(:,1) == A0(:);
%    t ... 1-by-N; times

% INIT variables and allocate memory for Max_steps steps
Ncomp = size(A0,1);     % number of rows of A0 (number of compartments)
A = zeros(Ncomp,Max_steps); % for tracking compartments
t = zeros(1,Max_steps);     % for time
m = 1;                      %start with the first step
A(:,1) = A0; %everything in the first (1) collumn of A will be set to A0
% main loop
while ((t(m)<tfin) && (m<=Max_steps))
    % (a) generate two randon numbers r1 and r2 uniformly  in (0,1)
    r = rand(1,2);
    % (b) Compute alpha0 = \sum_{proc=1}^Nproc alpha_proc(t),
    %     where alpha_proc(t) = propensity function of proc-th process
    alpha = getAlpha(A(:,m)); % get alpha
    a0 = sum(alpha);
    % (c) The next process takes place at time t+tau, where
    %     tau = 1/alpha0 * log(1/r1)   ... natural logrithm
    if a0 > 0 % if any process is going to happen
        t_next = t(m) + 1/a0 * log(1/r(1)); % determine when
    else % no process is going to happen (now and from now on)
        t_next = tfin; % go to the time limit
    end; %if a0>0
    if t_next>=tfin  % if next process is to occur after the time limit
        t(m+1)=tfin; % it will not occur at all
        A(:,m+1)=A(:,m);
    else %next process within the time limit
        t(m+1) = t_next; % increase the time
        % (d) Determine which process happens. Find proc such that
        %     r2 >= 1/a0 \sum_{i=1}^{proc-1} alpha_i(t)
        %     r2 <  1/a0 \sum_{i=1}^proc     alpha_i(t)
        proc =  find((cumsum(alpha) >= r(2)*a0),1,'first'); % which one
        %     Update the numbers in each compartment
        A(:,m+1) = A(:,m) - e(:,proc) + p(:,proc); % let it happen
    end; %if t_next>=tfin
    m = m + 1; % prepare for the next step
end % while
A = A(:,1:m); % cut zeros at the end of arrays
t = t(1:m);   % cut zeros at the end of arrays
end %Gillespie

end