% this script identifies all ESS of a given matrix game
% user has to specify the payoff matrix A
% the script uses Haigh procedure. It works for generic A.

% However, the script seems to work even for non-generic matrices.
% MATLAB (or the script) displays a warning when non-genericity can 
% can cause problems.  For non-generic matrices, rather than Haigh
% procedure, testing for ESS by random invading strategies was implemented

%% Input of payoff matrix A
 A = [ 0  -1   2  -3   2;
      2   0  -1   3  -1 ;
     -1   2   0   2  -1;
     -8   7   1   0   1;
     -1   2  -1   2   0];
 
%% Some init for the algorithm
N=size(A,1);    % number of strategies
all_s=1:N;      % list of all strategies
S=eye(N);       % identity matrix used for getting pure strategies
                % S(i,:) will be pure ith strategy    
ESS=[];         % so far the list of ESSs is empty
MaxTrials = 100;% number of invasion trials we will do if we cannot 
                % do Haigh procedure. 100 is more than enough for 5x5
                % matrices (or smaller)
                 
%% Actual algorithm
for ind= 2:2^N % go over almost all 2^N subsets of {1, 2, ..., N} 
    % start from 2 because at least one strategy must be involved     
    % convert ind to binary string (and flip it for easier reading
    support=fliplr(dec2bin(ind-1, N)=='1');
    SpEqualsTp = 1;     % assume supp(p)=T(p); (true if A is generic)
    CanBeESS = 1;       % first assume that it is not an ESS
    involved_strategies=all_s(support);
    n=length(involved_strategies);    % number of strategies in support
    first_involved=involved_strategies(1);
    if n==1 % support has only 1 element
        p = S(support,:); %so it is a pure strategy
    else % support has several elements and needs to solve E[S_i,p]=E[p,p]
        E=-S; % this will put -1 on the diagonal, -1 will stay there
              % in rows not in the desired support of p, 
              % which in turn will force p to be 0 outside of its support
        for s=all_s(support) % s is in the support of p
            % this will be for E[Si-Ss,p]=0, i.e. ``(Si-Ss)*A*p'=0''
            E(s,:)=(E(s,:)+S(first_involved,:))*A;
        end
        B=zeros(N,1);           % init right side of the equation
        B(first_involved)=1;    % elements of p need to add to 1
        E(first_involved,:) = ones(N,1); % forces p to add to 1
        p=(E\B)';               % p solves E*p'=B
        % due to rounding, p may be a slightly off zero outside of support
        p=p.*support;           % this will force p to live within support
        % when A is not generic, E can be singular and consequently p can
        % contain NaN or Inf or -Inf
        % MATLAB displays warning about singularity 
        % and the coding all(p(support)>0) below treats it as no ESS in
        % such a case
        CanBeESS= all(p(support)>0); % ESS must have positive coordinates
    end; % if n==1
    if CanBeESS==1 % if we still think p is an ESS
        % we try to invade by pure stratgegies not in the support of p
        for s=all_s(~support)       % for all s not in the support of p
            if (S(s,:)-p)*A*p'>0    % if s can invade
                CanBeESS=0;         % p is not an ESS
            elseif (S(s,:)-p)*A*p'==0 % this can happen in nongeneric game
                % p can still be ESS but support of p differs from T(p)
                support(s)=1; %extend the support for subsequent testing
                if SpEqualsTp==1
                    % if we thought so far that S(p)=T(p)
                    % display a warning and update our knowledge
                    disp('Matrix is non-generic');
                    SpEqualsTp=0; % S(p) is not T(p)
                end;
            end; % s can invade
        end; % for s not in the support of p
    end; % if CanBeESS
    if CanBeESS==1 % if we still think cp is an ESS
        if SpEqualsTp==1 % p satisfies S(p)=T(p), do Haigh procedure
            for k=1:n % get through all involved_strategies
                %define the aij-aik-akj+akk matrix
                H=zeros(n-1); % initialize 
                sk=involved_strategies(k); % get the strategy S(k)
                for ii=[1:k-1 k+1:n] % for all i~=k
                    si=involved_strategies(ii); % get the strategy
                    for j=[1:k-1 k+1:n] % for all j~=k
                        sj=involved_strategies(j); %get the strategy
                        % we need to skip index k which we will do as
                        % follows: i-(i>k) = i if i<k and =i-1 if i>k
                        H(ii-(ii>k),j-(j>k))=A(si,sj)-A(si,sk)- ...
                                                A(sk,sj)+A(sk,sk);
                    end
                end
                [dummy,temp]=chol(-(H+H')); % temp=0 if -H is positive definite
                if temp~=0 % H is not negative definite
                    CanBeESS=0; % p is not an ESS
                end
            end; % for k
        else % T(p) is not S(p), test p by random invaders
            Trial=0; % initial count of invasions
            while Trial<MaxTrials && CanBeESS==1
                for ind_q= 2:2^N % go over almost all 2^N subsets of {1, 2, ..., N} 
                    q=zeros(1,N); % init
                    % start from 2 because at least one strategy must be involved
                    % convert ind to binary string (and flip it for easier reading
                    invading_supp = support & fliplr(dec2bin(ind_q-1, N)=='1');
                    if any(invading_supp==1)
                        n = sum((invading_supp)); % regenerate as we added to support
                        if n==1
                            q(invading_supp)=1;
                        else
                            points=rand(1,n-1);
                            sorted_points = [0, sort(points,2),1];
                            q(invading_supp) = diff(sorted_points,[],2);
                        end
                    end
                    if ((p-q)*A*q'<=0) && ~all(q==p)
                        % q invaded p. Especially for small A, we need to make
                        % sure that q is not p which is the second condition
                        CanBeESS=0;
                    end;    
                end
                Trial=Trial+1; % increase the count
            end; % while InvasionTrials
        end; % if SpEqualsTp
    end; % if CanBeESS
    if CanBeESS==1
        CurNumESS=size(ESS,1)+1; % get the current count of the ESSs
        ESS(CurNumESS,:)=p;
    end
end
%% outputs
if size(ESS,1)>0 %if there is ESS
    disp(ESS)
else
    disp('No ESS')
end;


